(* Time-stamp: <modified the 22/03/2013 (at 08:35) by Erwan Jahier> *)

(** Manipulating data in the Soc interpreter  *)

type t = | I of int | F of float | B of bool | E of Soc.ident * int
         | A of t array 
         | S of (Soc.ident * t) list
         | U (* to set uninitialized mem *)

type path = Soc.ident list
type subst = (path * t)

(* Not really a good name anymore. Memory ? *)
type substs 


(* update the memory. The path correspond to a path in the call tree,
   except the last ident that is the variable name we want to
   associate a value to.
*)
val sadd : substs -> path -> t -> substs

(* ditto, but in the case of a partial assignment (e.g., a[i] = something).  *)
val sadd_partial : substs  -> Soc.var_expr -> path -> t -> substs


type ctx = { 
  cpath:Soc.ident list;
  s:substs;
}

(* Performs a recursive traversal of the top-level soc to init memories. *)
val create_ctx : Soc.tbl -> Soc.t -> ctx


val get_val : Soc.ident -> ctx -> t
val get_value : ctx -> Soc.var_expr -> t
val get_enum : Soc.ident -> ctx -> Soc.ident

(* Pretty-printers *)
val to_string  : t -> string
val string_of_substs :substs -> string

(* RIF I/O *)
val dump_substs : substs -> unit
val read_enum : Soc.ident list -> Soc.ident
val read_value : Soc.var -> t

(* if 
   args = [Var("x",Int); Const("3.14",Real) ] 
   pars = [("a",Int); ("b",Real)]
   s = [ "x",I(42) ; ... ]

then I want to output the follwing substitution :

   s = [ "a",I(42) ; "b",R(3.14) ]

nb : args and pars order matters

*)
val substitute_args_and_params : Soc.var_expr list -> Soc.var list -> ctx -> substs
val substitute_params_and_args : Soc.var list -> Soc.var_expr list -> ctx -> substs

(* Returns the top-level variable substitutions in a RIF format *)
val filter_top_subst : substs -> (Soc.ident * t) list
